package com.example.zdrojaknotifier;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.database.Cursor;
import android.support.v4.app.NotificationCompat;

public class NotificationReceiver extends BroadcastReceiver {
	
	private static final int NOTIFICATION_ID = 1;

	@Override
	public void onReceive(Context context, Intent incoming) {
		NotificationManager notificationManager = (NotificationManager) context
				.getSystemService(Context.NOTIFICATION_SERVICE);
		
		notificationManager.cancel(NOTIFICATION_ID);
		
		
		long lastViewed = context.getSharedPreferences(
				ZdrojakActivity.SHARED_PREFS_NAME, 0).getLong(
				ZdrojakActivity.LAST_VIEWED, -1);

		ContentResolver cr = context.getContentResolver();

		Cursor totalyNew = cr.query(ZdrojakContract.CONTENT_URI_ARTICLES,
				ZdrojakContract.PROJECTION, ZdrojakContract.ADDED + " > ?",
				new String[] { String.valueOf(lastViewed) },
				ZdrojakContract.SORT_ORDER_PUBDATE);

		Cursor unread = cr.query(ZdrojakContract.CONTENT_URI_ARTICLES,
				new String[] { ZdrojakContract.TITLE }, ZdrojakContract.VIEWED
						+ " = 0", null, ZdrojakContract.SORT_ORDER_PUBDATE);

		int unreadCount = unread.getCount();

		Notification n;
		NotificationCompat.Builder builder = new NotificationCompat.Builder(
				context);

		Intent intent = new Intent(context, ZdrojakActivity.class);

		PendingIntent pi = PendingIntent.getActivity(context, 0, intent, 0);

		if (totalyNew.getCount() == 1) {
			int titleIndex = totalyNew.getColumnIndex(ZdrojakContract.TITLE);
			int authorIndex = totalyNew.getColumnIndex(ZdrojakContract.AUTHOR);
			int descIndex = totalyNew
					.getColumnIndex(ZdrojakContract.DESCRIPTION);

			totalyNew.moveToFirst();

			n = builder
					.setContentTitle(totalyNew.getString(titleIndex))
					.setContentText(totalyNew.getString(authorIndex))
					.setSmallIcon(R.drawable.notification)
					.setContentIntent(pi)
					.setNumber(unreadCount)
					.setStyle(
							new NotificationCompat.BigTextStyle()
									.bigText(totalyNew.getString(descIndex)))
					.build();
		} else if (totalyNew.getCount() > 1) {
			Resources res = context.getResources();
			int titleIndex = unread.getColumnIndex(ZdrojakContract.TITLE);
			
			NotificationCompat.InboxStyle st = new NotificationCompat.InboxStyle();
			
			if(unread.getCount() > 5)
				st.setSummaryText(res.getString(R.string.and_x_more, unread.getCount() - 5));
			
			for(int i = 0;i < 5 && unread.moveToNext();i++){
				st.addLine(unread.getString(titleIndex));
			}
			
			n = builder
					.setContentTitle(res.getString(R.string.new_articles))
					.setContentText(
							res.getString(R.string.new_articles_count,
									unreadCount))
					.setSmallIcon(R.drawable.notification)
					.setContentIntent(pi)
					.setNumber(unreadCount)
					.setStyle(st)
					.build();
		} else{
			return;
		}

		notificationManager.notify(NOTIFICATION_ID, n);
	}
}
