package com.example.zdrojaknotifier;

import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import android.content.ContentValues;

public class RSSParser {

	private String rss;
	private Map<String, String> keys;
	private SimpleDateFormat format;

	public RSSParser(String rss) {
		this.rss = rss;

		format = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss z", Locale.US);

		keys = new HashMap<String, String>();

		keys.put("title", ZdrojakContract.TITLE);
		keys.put("link", ZdrojakContract.LINK);
		keys.put("description", ZdrojakContract.DESCRIPTION);
		keys.put("author", ZdrojakContract.AUTHOR);
		keys.put("pubDate", ZdrojakContract.PUBDATE);
	}

	public ContentValues[] parse() throws RSSParserException {
		Document doc = getDOM(rss);

		List<ContentValues> list = new ArrayList<ContentValues>();

		NodeList items = doc.getElementsByTagName("item");

		Node item;
		ContentValues values;
		NodeList children;
		Node child;
		for (int i = 0; (item = items.item(i)) != null; i++) {
			values = new ContentValues();
			children = item.getChildNodes();

			for (int j = 0; (child = children.item(j)) != null; j++) {
				if (child.getNodeType() == Node.ELEMENT_NODE)
					processNode(values, child);
			}

			list.add(values);
		}

		return list.toArray(new ContentValues[0]);
	}

	private Document getDOM(String rss) throws RSSParserException {
		Document doc = null;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {

			DocumentBuilder db = dbf.newDocumentBuilder();

			InputSource is = new InputSource();
			is.setCharacterStream(new StringReader(rss));
			doc = db.parse(is);
			return doc;

		} catch (ParserConfigurationException e) {
		} catch (SAXException e) {
		} catch (IOException e) {
		}

		throw new RSSParserException();
	}

	private void processNode(ContentValues values, Node node) {
		String key = getKeyForTagName(node.getNodeName());

		if (key == null)
			return;

		String nodeValue = getNodeTextContent(node);
		if (key.equals(ZdrojakContract.PUBDATE)) {
			long pubDate = 0;
			try {
				pubDate = format.parse(nodeValue).getTime();
			} catch (ParseException e) {
				e.printStackTrace();
			}
			values.put(key, pubDate);
		} else if (key.equals(ZdrojakContract.AUTHOR)) {
			String authorName = nodeValue.replaceFirst(
					"redakce@zdrojak\\.cz \\(Zdroják: (.*)\\).*", "$1");
			values.put(key, authorName);
		} else {
			values.put(key, nodeValue);
		}
	}

	private String getKeyForTagName(String tagName) {
		if (keys.containsKey(tagName))
			return keys.get(tagName);
		else
			return null;
	}

	private String getNodeTextContent(Node node) {
		String ret = "";

		NodeList children = node.getChildNodes();
		Node child;
		for (int i = 0; (child = children.item(i)) != null; i++) {
			if (child.getNodeType() == Node.TEXT_NODE) {
				if (!ret.equals(""))
					ret += " ";
				ret += child.getNodeValue();
			}
		}

		return ret;
	}
}
