package com.example.zdrojaknotifier;

import java.util.Calendar;
import java.util.Date;

import android.app.AlarmManager;
import android.app.ListActivity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ListView;

public class ZdrojakActivity extends ListActivity {

	private BroadcastReceiver receiver;
	private ArticlesAdapter adapter = null;
	public static final String SHARED_PREFS_NAME = "sharedPrefs";
	public static final String LAST_VIEWED = "lastViewed";

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Pro testování - Pozor na updatedToday = true, data se nemusí stáhnout!
//		getContentResolver().delete(ZdrojakContract.CONTENT_URI_ARTICLES, "1",
//				null);

		updateDataIfNone();
		updateList();
		setupDataUpdating();

		IntentFilter filter = new IntentFilter();
		filter.addAction(ZdrojakContract.ACTION_ZDROJAK_UPDATED);

		receiver = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				updateList();
			}
		};

		registerReceiver(receiver, filter);
	}

	@Override
	public void onResume() {
		super.onResume();
		setLastViewedNow();
	}

	@Override
	public void onDestroy() {
		unregisterReceiver(receiver);
		super.onDestroy();
	}

	private void updateList() {
		Cursor c = getContentResolver().query(
				ZdrojakContract.CONTENT_URI_ARTICLES,
				ZdrojakContract.PROJECTION, null, null,
				ZdrojakContract.SORT_ORDER_PUBDATE);

		if (adapter == null) {
			adapter = new ArticlesAdapter(this, c, 0);
			setListAdapter(adapter);
		} else {
			adapter.swapCursor(c);
		}

	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		String uriString = (String) v.getTag(R.id.tag_link);
		ZdrojakContract.setViewed(this, id, true);
		Intent i = new Intent(Intent.ACTION_VIEW, Uri.parse(uriString));
		startActivity(i);
	}

	private void setLastViewedNow() {
		getSharedPreferences(SHARED_PREFS_NAME, 0).edit()
				.putLong(LAST_VIEWED, new Date().getTime()).commit();
		sendBroadcast(new Intent(ZdrojakContract.ACTION_LASTVIEW_CHANGED));
	}

	private void setupDataUpdating() {
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(System.currentTimeMillis());
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MINUTE, 30);
		calendar.set(Calendar.HOUR, 0);
		calendar.set(Calendar.AM_PM, Calendar.AM);
		calendar.add(Calendar.DAY_OF_MONTH, 1);

		Intent intent = new Intent(this, UpdateReceiver.class);
		intent.putExtra(UpdateReceiver.ALARM_BROADCAST, true);
		PendingIntent pi = PendingIntent.getBroadcast(this, 0, intent, 0);

		((AlarmManager) getSystemService(Context.ALARM_SERVICE)).setRepeating(
				AlarmManager.RTC, calendar.getTimeInMillis(),
				AlarmManager.INTERVAL_DAY, pi);
	}

	private void updateDataIfNone() {
		Cursor c = getContentResolver().query(
				ZdrojakContract.CONTENT_URI_ARTICLES,
				ZdrojakContract.PROJECTION, null, null,
				ZdrojakContract.DEFAULT_SORT_ORDER);

		if (c.getCount() == 0) {
			Intent intent = new Intent(this, UpdateReceiver.class);
			sendBroadcast(intent);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_mark_all_read:
			ContentValues values = new ContentValues();
			values.put(ZdrojakContract.VIEWED, true);
			getContentResolver().update(ZdrojakContract.CONTENT_URI_ARTICLES,
					values, "1", null);
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}
}
